@@ -1,12 +1,10 @@
 import json
 import os
 
-import pytest
 from tktl.commands.login import LogInCommand, SetApiKeyCommand
-from tktl.core.exceptions import InvalidInputError
 
 
-def test_set_api_key(capsys):
+def test_set_api_key_command(capsys):
     cmd = SetApiKeyCommand()
     cmd.execute(api_key=None)
     out, err = capsys.readouterr()
@@ -19,11 +17,21 @@ def test_set_api_key(capsys):
         assert d["api-key"] == "ABC"
 
 
-def test_login(capsys):
+def test_login_command(capsys):
     cmd = LogInCommand()
     k = os.environ["TEST_USER_API_KEY"]
     assert cmd.execute(k) is True
     out, err = capsys.readouterr()
     assert out == f"Authentication successful for user: {os.environ['TEST_USER']}\n"
-    with pytest.raises(InvalidInputError):
-        cmd.execute(None)
+    assert cmd.execute(None) is False
+
+
+def test_login_fail_command(capfd):
+    cmd = LogInCommand()
+    assert cmd.execute() is False
+    out, err = capfd.readouterr()
+    assert err == "Authentication failed: no key provided\n"
+
+    assert cmd.execute(api_key="INVALID_UUID") is False
+    out, err = capfd.readouterr()
+    assert err == "Authentication failed: Key format is invalid\n"
