@@ -4,14 +4,6 @@ import pytest
 
 from tktl.login import login, logout
 
-pytest_plugins = [
-    "tests.cli",
-    "tests.sdk",
-    "tests.clients",
-    "tests.managers",
-    "tests.commands",
-]
-
 
 @pytest.fixture
 def user_key():
@@ -21,9 +13,9 @@ def user_key():
 @pytest.fixture
 def test_user_deployed_repos():
     return (
-        ("tktl-admin/sample-project", "master", "repayment"),
         ("tktl-admin/other-project", "testbranch", "repayment"),
         ("tktl-admin/other-project", "master", "repayment"),
+        ("tktl-admin/sample-project", "master", "repayment"),
         ("tktl-admin/integ-testing", "master", "repayment"),
         ("tktl-admin/grpc-test", "main", "repayment"),
     )
@@ -33,3 +25,8 @@ def test_user_deployed_repos():
 def logged_in_context():
     yield login(os.environ["TEST_USER_API_KEY"])
     logout()
+
+
+@pytest.fixture()
+def sample_deployed_url():
+    return "tktl-adm-grpc-test-main-1a485d0d54.stg-saas.taktile.com"
