@@ -1,4 +1,4 @@
-from typing import Callable, List
+from typing import Any, Callable, List
 
 import numpy as np
 import pandas as pd
@@ -56,11 +56,13 @@ def test_tabular_kinds(kind):
 def test_custom_kind():
     tktl = Tktl()
 
-    with pytest.raises(ValueError):
-        # Must define either payload or X, y
-        @tktl.endpoint(kind="custom")
-        def predict_untyped(x):
-            return x
+    @tktl.endpoint(kind="custom")
+    def predict_untyped(x):
+        return x
+
+    @tktl.endpoint(kind="custom", payload_model=Any, response_model=Any)
+    def predict_untyped_any(x):
+        return x
 
     class Payload(BaseModel):
         a: List
@@ -71,7 +73,7 @@ def test_custom_kind():
         c: List
 
     @tktl.endpoint(kind="custom", payload_model=Payload, response_model=Response)
-    def predict_typed(x):
+    def predict_typed_input_schemas(x):
         return x
 
     for endpoint in tktl.endpoints:
@@ -82,11 +84,11 @@ def test_custom_kind():
     y = pd.Series([1, 2, 3])
 
     @tktl.endpoint(kind="custom", X=x, y=y)
-    def predict_typed(x):
+    def predict_typed_input_frames(x):
         return x
 
-    schema = tktl.endpoints[1].input_schema.pydantic.schema()
-    assert schema["title"] == "DataFrame__predict_typed__input"
+    schema = tktl.endpoints[3].input_schema.pydantic.schema()
+    assert schema["title"] == "DataFrame__predict_typed_input_frames__input"
     assert schema["example"] == [{"a": 1, "b": 2}]
 
 
