@@ -1,4 +1,7 @@
-from docker.errors import APIError
+import os
+from typing import Dict
+
+from docker.errors import APIError  # type: ignore
 from pydantic import ValidationError
 
 from tktl.core.exceptions.exceptions import (
@@ -44,14 +47,11 @@ def log_failure(validation_output: ProjectValidationOutput):
     LOG.log(validation_output.text, color="red", err=True)
 
 
-def build_image(path: str, nocache: bool = False):
+def build_image(path: str, nocache: bool = False, secrets: Dict[str, str] = None):
     dm = DockerManager(path)
-
-    LOG.log("Patching docker file...")
-    dm.patch_docker_file()
     LOG.log("Building docker image...")
-    image = dm.build_image(nocache=nocache)
-    dm.remove_patched_docker_file()
+    abs_path = os.path.abspath(os.path.join(path, ".buildfile"))
+    image = dm.build_image(dockerfile=abs_path, nocache=nocache, buildargs=secrets)
 
     return dm, image
 
@@ -62,11 +62,12 @@ def status_output(status, logs):
     else:
         LOG.log("Error", color="red")
         print(logs.decode())
+        exit(1)
 
 
-def validate_import(path: str, nocache: bool):
+def validate_import(path: str, nocache: bool, secrets: Dict[str, str] = None):
     try:
-        dm, image = build_image(path, nocache=nocache)
+        dm, image = build_image(path, nocache=nocache, secrets=secrets)
         LOG.log("Testing import of tktl...")
         status, logs = dm.test_import(image)
         status_output(status, logs)
@@ -77,9 +78,9 @@ def validate_import(path: str, nocache: bool):
         )
 
 
-def validate_unittest(path: str, nocache: bool):
+def validate_unittest(path: str, nocache: bool, secrets: Dict[str, str] = None):
     try:
-        dm, image = build_image(path=path, nocache=nocache)
+        dm, image = build_image(path=path, nocache=nocache, secrets=secrets)
         LOG.log("Running unittests...")
         status, logs = dm.test_unittest(image)
         status_output(status, logs)
@@ -90,23 +91,36 @@ def validate_unittest(path: str, nocache: bool):
         )
 
 
-def validate_integration(path: str, nocache: bool):
+def validate_integration(
+    path: str, nocache: bool, timeout: int, retries: int, secrets: Dict[str, str] = None
+):
     try:
-        dm, image = build_image(path, nocache=nocache)
+        dm, image = build_image(path, nocache=nocache, secrets=secrets)
         LOG.log("Waiting for service to start...")
-        rest_response, grpc_response, container = dm.run_and_check_health(
-            image, kill_on_success=True
+        (
+            rest_response,
+            grpc_response,
+            arrow_container,
+            rest_container,
+        ) = dm.run_and_check_health(
+            image,
+            kill_on_success=True,
+            auth_enabled=False,
+            timeout=timeout,
+            retries=retries,
         )
         if _validate_container_response(
             rest_response=rest_response,
             grpc_response=grpc_response,
-            output=container.logs(),
+            rest_container_output=rest_container.logs(),
+            arrow_container_output=arrow_container.logs(),
         ):
             LOG.log("Success", color="green")
         else:
             LOG.log(
                 "Unable to run container. See stack trace for more info", color="red"
             )
+            exit(1)
 
     except MissingDocker:
         LOG.log(
@@ -115,32 +129,45 @@ def validate_integration(path: str, nocache: bool):
         )
 
 
-def validate_profiling(path: str, nocache: bool):
+def validate_profiling(
+    path: str, nocache: bool, timeout: int, retries: int, secrets: Dict[str, str] = None
+):
     try:
-        dm, image = build_image(path, nocache=nocache)
+        dm, image = build_image(path, nocache=nocache, secrets=secrets)
         LOG.log("Testing profiling...")
         LOG.log("Initiating service...")
-        rest_response, grpc_response, container = dm.run_and_check_health(
-            image, kill_on_success=False, auth_enabled=False
+        (
+            rest_response,
+            grpc_response,
+            arrow_container,
+            rest_container,
+        ) = dm.run_and_check_health(
+            image,
+            kill_on_success=False,
+            auth_enabled=False,
+            timeout=timeout,
+            retries=retries,
         )
         success = _validate_container_response(
             rest_response=rest_response,
             grpc_response=grpc_response,
-            output=container.logs(),
+            rest_container_output=rest_container.logs(),
+            arrow_container_output=arrow_container.logs(),
         )
         if not success:
             LOG.log(
                 "Failed to run service container, ensure service can run with `tktl validate integration`",
                 color="red",
             )
-            return
+            exit(1)
         try:
             LOG.log("Initiating remote profiling...")
             status, logs = dm.run_profiling_container()
             status_output(status, logs)
         finally:
             try:
-                container.kill()
+                arrow_container.kill()
+                rest_container.kill()
             except APIError:
                 pass
 
@@ -151,23 +178,46 @@ def validate_profiling(path: str, nocache: bool):
         )
 
 
-def _validate_container_response(rest_response, grpc_response, output):
+def _validate_rest_response(rest_response, output):
     if rest_response is None:
         LOG.log("Could not access REST endpoint", color="red")
         print(output.decode())
-    elif rest_response.status_code != 200:
+    elif rest_response.status_code != 204:
         LOG.log(f"Response status code {rest_response.status_code}", color="red")
         print(output.decode())
-    elif grpc_response is None:
-        LOG.log("Could not access gRPC endpoint", color="red")
     else:
         return True
     return False
 
 
-def validate_all(path: str, nocache: bool):
+def _validate_grpc_response(grpc_response, output):
+    if grpc_response is None:
+        LOG.log("Could not access gRPC endpoint", color="red")
+        print(output.decode())
+    else:
+        return True
+    return False
+
+
+def _validate_container_response(
+    rest_response, grpc_response, arrow_container_output, rest_container_output
+):
+    return _validate_grpc_response(
+        grpc_response=grpc_response, output=arrow_container_output
+    ) and _validate_rest_response(
+        rest_response=rest_response, output=rest_container_output
+    )
+
+
+def validate_all(
+    path: str, nocache: bool, timeout: int, retries: int, secrets: Dict[str, str] = None
+):
     validate_project_config(path)
-    validate_import(path, nocache=nocache)
-    validate_unittest(path, nocache=False)
-    validate_integration(path, nocache=False)
-    validate_profiling(path, nocache=False)
+    validate_import(path, nocache=nocache, secrets=secrets)
+    validate_unittest(path, nocache=False, secrets=secrets)
+    validate_integration(
+        path, nocache=False, timeout=timeout, retries=retries, secrets=secrets
+    )
+    validate_profiling(
+        path, nocache=False, timeout=timeout, retries=retries, secrets=secrets
+    )
