@@ -1,6 +1,6 @@
 from typing import Any
 
-import pandas
+import pandas  # type: ignore
 
 
 def pandas_to_python_type(series: pandas.Series):
